# Generated by roxygen2: do not edit by hand

S3method("$",dataquieR_result)
S3method("$",dataquieR_resultset2)
S3method("$",dq_lazy_ggplot)
S3method("$",dq_lazy_ggplot_s7)
S3method("$<-",dataquieR_resultset2)
S3method("%%",numeric_with_unit)
S3method("%/%",numeric_with_unit)
S3method("&",dq_lazy_ggplot)
S3method("*",dq_lazy_ggplot)
S3method("*",numeric_with_unit)
S3method("+",dq_lazy_ggplot)
S3method("+",numeric_with_unit)
S3method("-",dq_lazy_ggplot)
S3method("-",numeric_with_unit)
S3method("/",dq_lazy_ggplot)
S3method("/",numeric_with_unit)
S3method("[",dataquieR_result)
S3method("[",dataquieR_resultset2)
S3method("[<-",dataquieR_resultset2)
S3method("[[",dataquieR_result)
S3method("[[",dataquieR_resultset2)
S3method("[[",dq_lazy_ggplot)
S3method("[[",dq_lazy_ggplot_s7)
S3method("[[<-",dataquieR_resultset2)
S3method("^",numeric_with_unit)
S3method("names<-",dataquieR_translated)
S3method("|",dq_lazy_ggplot)
S3method(as.character,dataquieR_translated)
S3method(as.character,interval)
S3method(as.data.frame,dataquieR_resultset)
S3method(as.list,dataquieR_resultset)
S3method(as.list,dataquieR_resultset2)
S3method(dim,dataquieR_resultset2)
S3method(dimnames,dataquieR_resultset2)
S3method(droplevels,ReportSummaryTable)
S3method(ggplot_build,dq_lazy_ggplot)
S3method(ggplot_build,dq_lazy_ggplot_s7)
S3method(grid::grid.draw,svg_plot_proxy)
S3method(grid::grid.draw,util_pairs_ggplot_panels)
S3method(knitr::knit_print,ReportSummaryTable)
S3method(plot,dataquieR_summary)
S3method(prep_extract_summary,dataquieR_result)
S3method(prep_extract_summary,dataquieR_resultset2)
S3method(print,DataSlot)
S3method(print,ReportSummaryTable)
S3method(print,Slot)
S3method(print,StudyDataSlot)
S3method(print,TableSlot)
S3method(print,dataquieR_result)
S3method(print,dataquieR_resultset)
S3method(print,dataquieR_resultset2)
S3method(print,dataquieR_summary)
S3method(print,dataquieR_translated)
S3method(print,dq_lazy_ggplot)
S3method(print,dq_lazy_ggplot_s7)
S3method(print,interval)
S3method(print,list)
S3method(print,master_result)
S3method(print,numeric_with_unit)
S3method(print,svg_plot_proxy)
S3method(print,util_pairs_ggplot_panels)
S3method(rbind,ReportSummaryTable)
S3method(resnames,dataquieR_resultset2)
S3method(summary,dataquieR_resultset)
S3method(summary,dataquieR_resultset2)
S3method(util_undisclose,Slot)
S3method(util_undisclose,data.frame)
S3method(util_undisclose,dataquieR_result)
S3method(util_undisclose,dataquieR_resultset2)
S3method(util_undisclose,default)
S3method(util_undisclose,dq_lazy_ggplot)
S3method(util_undisclose,dq_lazy_ggplot_s7)
S3method(util_undisclose,gg)
S3method(util_undisclose,ggmatrix_fn_with_params)
S3method(util_undisclose,ggmatrix_plot_obj)
S3method(util_undisclose,ggplot_built)
S3method(util_undisclose,list)
S3method(util_undisclose,svg_plot_proxy)
S3method(util_undisclose,util_pairs_ggplot_panels)
export(.get_internal_api)
export(API_VERSION)
export(ASSOCIATION_DIRECTION)
export(ASSOCIATION_FORM)
export(ASSOCIATION_METRIC)
export(ASSOCIATION_RANGE)
export(CHECK_ID)
export(CHECK_LABEL)
export(CODE_CLASS)
export(CODE_CLASSES)
export(CODE_INTERPRET)
export(CODE_LABEL)
export(CODE_LIST_TABLE)
export(CODE_ORDER)
export(CODE_VALUE)
export(COMPUTATION_RULE)
export(CONTRADICTION_TERM)
export(CONTRADICTION_TYPE)
export(DATA_PREPARATION)
export(DATA_TYPES)
export(DATA_TYPES_OF_R_TYPE)
export(DF_CODE)
export(DF_ELEMENT_COUNT)
export(DF_ID_REF_TABLE)
export(DF_ID_VARS)
export(DF_NAME)
export(DF_RECORD_CHECK)
export(DF_RECORD_COUNT)
export(DF_UNIQUE_ID)
export(DF_UNIQUE_ROWS)
export(DISTRIBUTIONS)
export(GOLDSTANDARD)
export(MAHALANOBIS_THRESHOLD)
export(MISSING_CODE_RULES)
export(MULTIVARIATE_OUTLIER_CHECK)
export(MULTIVARIATE_OUTLIER_CHECKTYPE)
export(REL_VAL)
export(RULE)
export(SCALE_ACRONYM)
export(SCALE_LEVELS)
export(SCALE_NAME)
export(SEGMENT_ID_REF_TABLE)
export(SEGMENT_ID_TABLE)
export(SEGMENT_ID_VARS)
export(SEGMENT_MISS)
export(SEGMENT_PART_VARS)
export(SEGMENT_RECORD_CHECK)
export(SEGMENT_RECORD_COUNT)
export(SEGMENT_UNIQUE_ID)
export(SEGMENT_UNIQUE_ROWS)
export(SPLIT_CHAR)
export(VARATT_REQUIRE_LEVELS)
export(VARIABLE_LIST)
export(VARIABLE_LIST_ORDER)
export(VARIABLE_ROLES)
export(WELL_KNOWN_META_VARIABLE_NAMES)
export(acc_cat_distributions)
export(acc_distributions)
export(acc_distributions_ecdf)
export(acc_distributions_loc)
export(acc_distributions_only)
export(acc_distributions_prop)
export(acc_end_digits)
export(acc_loess)
export(acc_mahalanobis)
export(acc_mahalanobis_ratio)
export(acc_margins)
export(acc_multivariate_outlier)
export(acc_robust_univariate_outlier)
export(acc_shape_or_scale)
export(acc_univariate_outlier)
export(acc_varcomp)
export(com_item_missingness)
export(com_qualified_item_missingness)
export(com_qualified_segment_missingness)
export(com_segment_missingness)
export(com_unit_missingness)
export(con_contradictions)
export(con_contradictions_redcap)
export(con_inadmissible_categorical)
export(con_inadmissible_vocabulary)
export(con_limit_deviations)
export(contradiction_functions)
export(contradiction_functions_descriptions)
export(dataquieR.applicability_problem)
export(dataquieR.intrinsic_applicability_problem)
export(dataquieR.study_data_cache_metrics_env_default)
export(des_scatterplot_matrix)
export(des_summary)
export(des_summary_categorical)
export(des_summary_continuous)
export(dq_report)
export(dq_report2)
export(dq_report_by)
export(ggplotGrob.dq_lazy_ggplot)
export(ggplotGrob.dq_lazy_ggplot_s7)
export(ggplotly.dq_lazy_ggplot)
export(ggplotly.dq_lazy_ggplot_s7)
export(html_dependency_jspdf)
export(int_all_datastructure_dataframe)
export(int_all_datastructure_segment)
export(int_datatype_matrix)
export(int_duplicate_content)
export(int_duplicate_ids)
export(int_encoding_errors)
export(int_part_vars_structure)
export(int_sts_element_dataframe)
export(int_sts_element_segment)
export(int_unexp_elements)
export(int_unexp_records_dataframe)
export(int_unexp_records_segment)
export(int_unexp_records_set)
export(nres)
export(pipeline_recursive_result)
export(pipeline_vectorized)
export(plotly_build.dq_lazy_ggplot)
export(plotly_build.dq_lazy_ggplot_s7)
export(prep_acc_distributions_with_ecdf)
export(prep_add_cause_label_df)
export(prep_add_computed_variables)
export(prep_add_data_frames)
export(prep_add_missing_codes)
export(prep_add_to_meta)
export(prep_apply_coding)
export(prep_check_for_dataquieR_updates)
export(prep_check_meta_data_dataframe)
export(prep_check_meta_data_segment)
export(prep_check_meta_names)
export(prep_clean_labels)
export(prep_combine_report_summaries)
export(prep_compare_meta_with_study)
export(prep_create_meta)
export(prep_create_meta_data_file)
export(prep_create_storr_factory)
export(prep_datatype_from_data)
export(prep_deparse_assignments)
export(prep_deregister_progress_hook)
export(prep_dq_data_type_of)
export(prep_expand_codes)
export(prep_extract_cause_label_df)
export(prep_extract_classes_by_functions)
export(prep_extract_summary)
export(prep_fix_meta_id_dups)
export(prep_get_data_frame)
export(prep_get_labels)
export(prep_get_study_data_segment)
export(prep_get_user_name)
export(prep_get_variant)
export(prep_guess_encoding)
export(prep_is_translated)
export(prep_link_escape)
export(prep_list_dataframes)
export(prep_list_voc)
export(prep_load_folder_with_metadata)
export(prep_load_report)
export(prep_load_report_from_backend)
export(prep_load_workbook_like_file)
export(prep_map_labels)
export(prep_merge_study_data)
export(prep_meta_data_v1_to_item_level_meta_data)
export(prep_min_obs_level)
export(prep_open_in_excel)
export(prep_pmap)
export(prep_prepare_dataframes)
export(prep_purge_data_frame_cache)
export(prep_realize_ggplot)
export(prep_register_progress_hook)
export(prep_remove_from_cache)
export(prep_render_pie_chart_from_summaryclasses_ggplot2)
export(prep_render_pie_chart_from_summaryclasses_plotly)
export(prep_robust_guess_data_type)
export(prep_save_report)
export(prep_scalelevel_from_data_and_metadata)
export(prep_set_backend)
export(prep_study2meta)
export(prep_summary_to_classes)
export(prep_title_escape)
export(prep_undisclose)
export(prep_unsplit_val_tabs)
export(prep_valuelabels_from_data)
export(pro_applicability_matrix)
export(resnames)
export(to_basic.GeomPointrangeRobust)
import(patchwork)
import(qmrparser)
importFrom(MASS,fitdistr)
importFrom(dplyr,"%>%")
importFrom(dplyr,recode)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotate)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,discrete_scale)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_count)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_errorbar)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_path)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_pointrange)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_alpha)
importFrom(ggplot2,scale_alpha_manual)
importFrom(ggplot2,scale_color_gradientn)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_fill_gradientn)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_size_continuous)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_datetime)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,stat_ecdf)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,vars)
importFrom(ggplot2,waiver)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylab)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,gray.colors)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rgb)
importFrom(graphics,barplot)
importFrom(graphics,hist)
importFrom(methods,new)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,caller_fn)
importFrom(stats,IQR)
importFrom(stats,aggregate)
importFrom(stats,alias)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,dgamma)
importFrom(stats,dnorm)
importFrom(stats,dunif)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,lowess)
importFrom(stats,mad)
importFrom(stats,mahalanobis)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pgamma)
importFrom(stats,pnorm)
importFrom(stats,poisson)
importFrom(stats,predict)
importFrom(stats,punif)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reorder)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,adist)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,osVersion)
importFrom(utils,packageName)
importFrom(utils,packageVersion)
importFrom(utils,tail)
