#' @title The `dataquieR` package about Data Quality in
#'        Epidemiological Research
#' @name dataquieR
#' @aliases dataquieR-package dataquieR-options
#' @docType package
#' @description
#' For a quick start please read [dq_report2] and maybe the vignettes
#' or the package's [website](https://dataquality.qihs.uni-greifswald.de/).
#'
#' # Options
#'
#' This package features the following `options()`:
#'
#' `r paste0(' - [', grep("^dataquieR\\.", names(options()), value = TRUE), ']', collapse = "\n")`
#'
#' @family options
#' @references
#' - [JOSS 2024](https://joss.theoj.org/papers/10.21105/joss.06581)
#' - [JOSS 2021](https://joss.theoj.org/papers/10.21105/joss.03093)
#' - [DQ_OBS data quality framework](https://www.researchsquare.com/article/rs-119457/v1)
#'
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
## usethis namespace: end
NULL
