#' Verify an object of class `r dataquieR_resultset_class`
#'
#' Deprecated
#'
#' @param ... Deprecated
#'
#' @return Deprecated
#'
dataquieR_resultset_verify <- function(...) {
  lifecycle::deprecate_stop("2.1.0", what = "dq_report()", with = "dq_report2()")
}
