test_that("util_backtickQuote works", {
  skip_on_cran()
  expect_equal(util_backtickQuote(""), "")
  expect_equal(ignore_attr = TRUE, util_backtickQuote(NA), NA_character_)
  nm <- paste(letters, LETTERS)
  t <- paste("c(", paste(util_backtickQuote(nm), collapse = ", "), ")")
  e <- parse(text = t)
  got <- eval(e, envir = as.list(setNames(LETTERS, nm = nm)))
  expect_equal(ignore_attr = TRUE, got, LETTERS)
})
