# test-int_sts_element_dataframe works

    list(DataframeTable = structure(list(DF_NAME = "https://dataquality.qihs.uni-greifswald.de/extdata/fortests/ship.RDS", 
        NUM_int_sts_element = 1L, PCT_int_sts_element = 3.03030303030303), class = "data.frame", row.names = c(NA, 
    -1L)), DataframeData = structure(list(DF_NAME = "https://dataquality.qihs.uni-greifswald.de/extdata/fortests/ship.RDS", 
        "Unexpected data element set (Number)" = 1L, "Unexpected data element set (Percentage (0 to 100))" = 3.03, 
        "Affected Elements" = "{ \"sex\" } ∉ md"), row.names = c(NA, 
    -1L), class = "data.frame"))

