# prep_compare_meta_with_study works

    list(pred = structure(list(VAR_NAMES = c("v00000", "v00001", 
    "v00002", "v00003", "v00004", "v00005", "v00006", "v00007", "v00008", 
    "v00009", "v00010", "v00011", "v00012", "v00013", "v00014", "v00015", 
    "v00016", "v00017", "v00018", "v00019", "v00020", "v00021", "v00022", 
    "v00023", "v00024", "v00025", "v00026", "v00027", "v00028", "v00029", 
    "v00030", "v00031", "v00032", "v00033", "v00034", "v00035", "v00036", 
    "v00037", "v00038", "v00039", "v00040", "v00041", "v00042", "v00103", 
    "v00109", "v01002", "v01003", "v01018", "v10000", "v20000", "v30000", 
    "v40000", "v50000"), LABEL = c("v00000", "v00001", "v00002", 
    "v00003", "v00004", "v00005", "v00006", "v00007", "v00008", "v00009", 
    "v00010", "v00011", "v00012", "v00013", "v00014", "v00015", "v00016", 
    "v00017", "v00018", "v00019", "v00020", "v00021", "v00022", "v00023", 
    "v00024", "v00025", "v00026", "v00027", "v00028", "v00029", "v00030", 
    "v00031", "v00032", "v00033", "v00034", "v00035", "v00036", "v00037", 
    "v00038", "v00039", "v00040", "v00041", "v00042", "v00103", "v00109", 
    "v01002", "v01003", "v01018", "v10000", "v20000", "v30000", "v40000", 
    "v50000"), DATA_TYPE = c("integer", "string", "integer", "integer", 
    "integer", "integer", "float", "integer", "string", "integer", 
    "integer", "string", "string", "datetime", "float", "integer", 
    "integer", "datetime", "integer", "integer", "integer", "integer", 
    "integer", "integer", "integer", "integer", "integer", "integer", 
    "integer", "integer", "integer", "integer", "string", "datetime", 
    "integer", "integer", "integer", "integer", "integer", "integer", 
    "integer", "integer", "datetime", "string", "integer", "integer", 
    "integer", "integer", "integer", "integer", "integer", "integer", 
    "integer"), SCALE_LEVEL = c("nominal", "na", "nominal", "ratio", 
    "ratio", "ratio", "ratio", "nominal", "nominal", "ratio", "nominal", 
    "nominal", "nominal", "interval", "ratio", "ratio", "nominal", 
    "interval", "nominal", "nominal", "nominal", "ordinal", "nominal", 
    "nominal", "nominal", "nominal", "ordinal", "ordinal", "nominal", 
    "nominal", "nominal", "ordinal", "nominal", "interval", "ordinal", 
    "ordinal", "ordinal", "ordinal", "ordinal", "ordinal", "ordinal", 
    "ordinal", "interval", "nominal", "nominal", "nominal", "ratio", 
    "nominal", "nominal", "nominal", "nominal", "nominal", "nominal"
    ), UNIT = c(NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_, NA_character_, 
    NA_character_, NA_character_, NA_character_, NA_character_), 
        VALUE_LABELS = c(NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_), MISSING_LIST_TABLE = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), HARD_LIMITS = c(NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_), CO_VARS = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), GROUP_VAR_OBSERVER = c(NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_), GROUP_VAR_DEVICE = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), TIME_VAR = c(NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_), PART_VAR = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), STUDY_SEGMENT = c(NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_), LONG_LABEL = c("v00000", "v00001", 
        "v00002", "v00003", "v00004", "v00005", "v00006", "v00007", 
        "v00008", "v00009", "v00010", "v00011", "v00012", "v00013", 
        "v00014", "v00015", "v00016", "v00017", "v00018", "v00019", 
        "v00020", "v00021", "v00022", "v00023", "v00024", "v00025", 
        "v00026", "v00027", "v00028", "v00029", "v00030", "v00031", 
        "v00032", "v00033", "v00034", "v00035", "v00036", "v00037", 
        "v00038", "v00039", "v00040", "v00041", "v00042", "v00103", 
        "v00109", "v01002", "v01003", "v01018", "v10000", "v20000", 
        "v30000", "v40000", "v50000"), LOCATION_RANGE = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), LOCATION_METRIC = c(NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_), PROPORTION_RANGE = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), UNIVARIATE_OUTLIER_CHECKTYPE = c(NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_), N_RULES = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), MISSING_LIST = c("|", "|", "|", "|", "99980|99981|99982|99983|99984|99985|99986|99987|99988|99989|99990|99991|99992|99993|99994|99995", 
        "99980|99981|99982|99983|99984|99985|99986|99987|99988|99989|99990|99991|99992|99993|99994|99995", 
        "99980|99983|99987|99988|99989|99990|99991|99992|99993|99994|99995", 
        "99980|99988|99989|99991|99993|99994|99995", "|", "99980|99981|99982|99983|99984|99985|99986|99988|99989|99990|99991|99992|99993|99994|99995", 
        "99980|99987", "|", "|", "|", "99980|99981|99982|99983|99984|99985|99986|99988|99989|99990|99991|99992|99994|99995", 
        "99980|99981|99982|99983|99984|99985|99986|99988|99989|99990|99991|99992|99994|99995", 
        "|", "|", "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "88880|99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "88880|99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "|", "|", "99980|99983|99988|99989|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99991|99993|99994|99995", "99980|99983|99988|99989|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99991|99993|99994|99995", "99980|99983|99988|99989|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99991|99993|99994|99995", "99980|99983|99988|99989|99991|99993|99994|99995", 
        "99980|99983|99988|99989|99991|99993|99994|99995", "|", "|", 
        "99980|99981|99982|99983|99984|99985|99986|99987|99988|99989|99990|99991|99992|99993|99994|99995", 
        "|", "|", "99980|99983|99988|99989|99990|99991|99993|99994|99995", 
        "|", "|", "|", "|", "|"), JUMP_LIST = c(NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_)), row.names = c("v00000", 
    "v00001", "v00002", "v00003", "v00004", "v00005", "v00006", "v00007", 
    "v00008", "v00009", "v00010", "v00011", "v00012", "v00013", "v00014", 
    "v00015", "v00016", "v00017", "v00018", "v00019", "v00020", "v00021", 
    "v00022", "v00023", "v00024", "v00025", "v00026", "v00027", "v00028", 
    "v00029", "v00030", "v00031", "v00032", "v00033", "v00034", "v00035", 
    "v00036", "v00037", "v00038", "v00039", "v00040", "v00041", "v00042", 
    "v00103", "v00109", "v01002", "v01003", "v01018", "v10000", "v20000", 
    "v30000", "v40000", "v50000"), class = "data.frame"), prov = structure(list(
        VAR_NAMES = c("v00000", "v00001", "v00002", "v00003", "v00004", 
        "v00005", "v00006", "v00007", "v00008", "v00009", "v00010", 
        "v00011", "v00012", "v00013", "v00014", "v00015", "v00016", 
        "v00017", "v00018", "v00019", "v00020", "v00021", "v00022", 
        "v00023", "v00024", "v00025", "v00026", "v00027", "v00028", 
        "v00029", "v00030", "v00031", "v00032", "v00033", "v00034", 
        "v00035", "v00036", "v00037", "v00038", "v00039", "v00040", 
        "v00041", "v00042", "v00103", "v00109", "v01002", "v01003", 
        "v01018", "v10000", "v20000", "v30000", "v40000", "v50000"
        ), LABEL = c("CENTER_0", "PSEUDO_ID", "SEX_0", "AGE_0", "SBP_0", 
        "DBP_0", "GLOBAL_HEALTH_VAS_0", "ASTHMA_0", "VO2_CAPCAT_0", 
        "ARM_CIRC_0", "ARM_CUFF_0", "USR_VO2_0", "USR_BP_0", "EXAM_DT_0", 
        "CRP_0", "BSG_0", "DEV_NO_0", "LAB_DT_0", "EDUCATION_0", 
        "FAM_STAT_0", "MARRIED_0", "N_CHILD_0", "EATING_PREFS_0", 
        "MEAT_CONS_0", "SMOKING_0", "SMOKE_SHOP_0", "N_INJURIES_0", 
        "N_BIRTH_0", "INCOME_GROUP_0", "PREGNANT_0", "MEDICATION_0", 
        "N_ATC_CODES_0", "USR_SOCDEM_0", "INT_DT_0", "ITEM_1_0", 
        "ITEM_2_0", "ITEM_3_0", "ITEM_4_0", "ITEM_5_0", "ITEM_6_0", 
        "ITEM_7_0", "ITEM_8_0", "QUEST_DT_0", "AGE_GROUP_0", "ARM_CIRC_DISC_0", 
        "SEX_1", "AGE_1", "EDUCATION_1", "PART_STUDY", "PART_PHYS_EXAM", 
        "PART_LAB", "PART_INTERVIEW", "PART_QUESTIONNAIRE"), DATA_TYPE = c("integer", 
        "string", "integer", "integer", "float", "float", "float", 
        "integer", "string", "float", "integer", "string", "string", 
        "datetime", "float", "float", "integer", "datetime", "integer", 
        "integer", "integer", "integer", "integer", "integer", "integer", 
        "integer", "integer", "integer", "integer", "integer", "integer", 
        "integer", "string", "datetime", "integer", "integer", "integer", 
        "integer", "integer", "integer", "integer", "integer", "datetime", 
        "string", "integer", "integer", "integer", "integer", "integer", 
        "integer", "integer", "integer", "integer"), SCALE_LEVEL = c("nominal", 
        "na", "nominal", "ratio", "ratio", "ratio", "ratio", "nominal", 
        "ordinal", "ratio", "ordinal", "nominal", "nominal", "interval", 
        "ratio", "ratio", "nominal", "interval", "ordinal", "nominal", 
        "nominal", "ratio", "nominal", "ordinal", "nominal", "ordinal", 
        "ratio", "ratio", "ordinal", "nominal", "nominal", "ratio", 
        "nominal", "interval", "ratio", "ratio", "ratio", "ratio", 
        "ratio", "ratio", "ratio", "ratio", "interval", "ordinal", 
        "ordinal", "nominal", "ratio", "ordinal", "nominal", "nominal", 
        "nominal", "nominal", "nominal"), STANDARDIZED_VOCABULARY_TABLE = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), MISSING_LIST_TABLE = c(NA, NA, NA, NA, "missing_table", 
        "missing_table", "missing_table", "missing_table", "missing_table", 
        "missing_table", "missing_table", "missing_table", "missing_table", 
        NA, "missing_table", "missing_table", NA, NA, "missing_table", 
        "missing_table", "missing_table", "missing_table", "missing_table", 
        "missing_table", "missing_table", "missing_table", "missing_table", 
        "missing_table", "missing_table", "missing_table", "missing_table", 
        "missing_table", "missing_table", NA, "missing_table", "missing_table", 
        "missing_table", "missing_table", "missing_table", "missing_table", 
        "missing_table", "missing_table", NA, NA, "missing_table", 
        NA, NA, "missing_table", NA, NA, NA, NA, NA), HARD_LIMITS = c(NA, 
        NA, NA, "[18;Inf)", "[80;180]", "[50;Inf)", "[0;10]", "[0;1]", 
        NA, "[0;Inf)", "[1;3]", NA, NA, "[2018-01-01 00:00:00 CET;)", 
        "[0;Inf)", "[0;100]", NA, "[2018-01-01 00:00:00 CET;)", "[0;6]", 
        NA, "[0;1]", NA, "[0;2]", "[0;4]", "[0;1]", "[0;4]", NA, 
        NA, NA, "[0;1]", "[0;1]", "[0;Inf]", NA, "[2018-01-01 00:00:00 CET;)", 
        "[0;10]", "[0;10]", "[0;10]", "[0;10]", "[0;10]", "[0;10]", 
        "[0;10]", "[0;10]", "[2018-01-01 00:00:00 CET;)", NA, "[1;3]", 
        NA, "[18;Inf)", "[0;6]", NA, NA, NA, NA, NA), DETECTION_LIMITS = c(NA, 
        NA, NA, NA, "[0;265]", "[0;265]", NA, NA, NA, NA, NA, NA, 
        NA, NA, "[0.16;Inf)", NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), 
        SOFT_LIMITS = c(NA, NA, NA, NA, "(90;170)", "(55;100)", "[1;9]", 
        NA, NA, "(0;60]", NA, NA, NA, NA, "[0.2;10)", "[0.2;30)", 
        NA, NA, NA, NA, NA, "[0;5]", NA, NA, NA, NA, "[0;10]", "[0;10]", 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), DISTRIBUTION = c(NA, 
        NA, NA, NA, "normal", "normal", "uniform", NA, NA, "normal", 
        NA, NA, NA, NA, "gamma", "normal", "uniform", NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA), DECIMALS = c(NA, NA, NA, NA, "0", "0", "1", 
        NA, NA, "0", NA, NA, NA, NA, "3", "0", NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA), GROUP_VAR_OBSERVER = c(NA, NA, NA, NA, "USR_BP_0", 
        "USR_BP_0", NA, NA, "USR_VO2_0", NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, "USR_SOCDEM_0", "USR_SOCDEM_0", "USR_SOCDEM_0", 
        "USR_SOCDEM_0", "USR_SOCDEM_0", "USR_SOCDEM_0", "USR_SOCDEM_0", 
        "USR_SOCDEM_0", "USR_SOCDEM_0", "USR_SOCDEM_0", "USR_SOCDEM_0", 
        "USR_SOCDEM_0", "USR_SOCDEM_0", "USR_SOCDEM_0", NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, "USR_SOCDEM_0", 
        NA, NA, NA, NA, NA), GROUP_VAR_DEVICE = c(NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, "ARM_CUFF_0", NA, NA, NA, NA, "DEV_NO_0", 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, "ARM_CUFF_0", 
        NA, NA, NA, NA, NA, NA, NA, NA), TIME_VAR = c(NA, NA, NA, 
        NA, "EXAM_DT_0", "EXAM_DT_0", NA, NA, "EXAM_DT_0", "EXAM_DT_0", 
        NA, NA, NA, NA, "LAB_DT_0", "LAB_DT_0", NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA), STUDY_SEGMENT = c("STUDY", "STUDY", "STUDY", "STUDY", 
        "PHYS_EXAM", "PHYS_EXAM", "PHYS_EXAM", "PHYS_EXAM", "PHYS_EXAM", 
        "PHYS_EXAM", "PHYS_EXAM", "PHYS_EXAM", "PHYS_EXAM", "PHYS_EXAM", 
        "LAB", "LAB", "LAB", "LAB", "INTERVIEW", "INTERVIEW", "INTERVIEW", 
        "INTERVIEW", "INTERVIEW", "INTERVIEW", "INTERVIEW", "INTERVIEW", 
        "INTERVIEW", "INTERVIEW", "INTERVIEW", "INTERVIEW", "INTERVIEW", 
        "INTERVIEW", "INTERVIEW", "INTERVIEW", "QUESTIONNAIRE", "QUESTIONNAIRE", 
        "QUESTIONNAIRE", "QUESTIONNAIRE", "QUESTIONNAIRE", "QUESTIONNAIRE", 
        "QUESTIONNAIRE", "QUESTIONNAIRE", "QUESTIONNAIRE", "STUDY", 
        "PHYS_EXAM", "STUDY", "STUDY", "INTERVIEW", "STUDY", "PHYS_EXAM", 
        "LAB", "INTERVIEW", "QUESTIONNAIRE"), PART_VAR = c("v10000", 
        "v10000", "v10000", "v10000", "v20000", "v20000", "v20000", 
        "v20000", "v20000", "v20000", "v20000", "v20000", "v20000", 
        "v20000", "v30000", "v30000", "v30000", "v30000", "v40000", 
        "v40000", "v40000", "v40000", "v40000", "v40000", "v40000", 
        "v40000", "v40000", "v40000", "v40000", "v40000", "v40000", 
        "v40000", "v40000", "v40000", "v50000", "v50000", "v50000", 
        "v50000", "v50000", "v50000", "v50000", "v50000", "v50000", 
        "v10000", "v20000", "v10000", "v10000", "v40000", "v10000", 
        "v10000", "v10000", "v10000", "v40000"), VARIABLE_ROLE = c("intro", 
        "intro", "intro", "intro", "primary", "primary", "primary", 
        "primary", "primary", "secondary", "process", "process", 
        "process", "process", "primary", "primary", "process", "process", 
        "primary", "primary", "primary", "primary", "primary", "primary", 
        "primary", "primary", "primary", "primary", "primary", "primary", 
        "primary", "primary", "process", "process", "primary", "primary", 
        "primary", "primary", "primary", "primary", "primary", "primary", 
        "process", "secondary", "secondary", "intro", "intro", "primary", 
        "intro", "intro", "intro", "intro", "intro"), VARIABLE_ORDER = c("1", 
        "2", "3", "4", "9", "10", "11", "12", "13", "14", "16", "17", 
        "18", "19", "21", "22", "23", "24", "26", "28", "29", "30", 
        "31", "32", "33", "34", "35", "36", "37", "38", "39", "40", 
        "41", "42", "44", "45", "46", "47", "48", "49", "50", "51", 
        "52", "5", "15", "7", "6", "27", "8", "20", "25", "43", "53"
        ), LONG_LABEL = c("Examination center", "Participant ID", 
        "Sex B/L", "Age B/L", "Systolic blood pressure", "Diastolic blood pressure", 
        "Self-reported global health", "Known asthma", "Aerobic capacity category", 
        "Upper arm circumference", "Upper arm circumference device", 
        "Aerobic capacity examiner", "Blood pressure examiner", "Examination date and time", 
        "C-reactive protein", "Erythrocyte sedimentation rate", "Device ID", 
        "Lab analysis date and time", "Highest educational level B/L", 
        "Marital status", "Currently married", "Number of children", 
        "Eating preferences", "Meat consumption", "Current smoker", 
        "Purchasing tobacco products", "Number of injuries", "Number of births", 
        "Income group", "Currently pregnant", "Medication use", "Number of ATC codes", 
        "Sociodemographics examiner", "Interview date and time", 
        "Item 1", "Item 2", "Item 3", "Item 4", "Item 5", "Item 6", 
        "Item 7", "Item 8", "Questionnaire date and time", "Age group B/L", 
        "Upper arm circumference cat", "Sex F/U", "Age F/U", "Highest educational level F/U", 
        "Study consent", "Physical exam consent", "Lab analysis consent", 
        "Interview consent", "Questionnaire consent"), ELEMENT_HOMOGENITY_CHECKTYPE = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), UNIVARIATE_OUTLIER_CHECKTYPE = c(NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, "Sigma_gap | Tukey", NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA), N_RULES = c("4", "4", "4", "4", 
        "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", 
        "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", 
        "2", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", 
        "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", "4", 
        "4"), LOCATION_METRIC = c(NA, NA, NA, NA, "Mean", "Mean", 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA), LOCATION_RANGE = c(NA, NA, NA, NA, "(100;140)", 
        "(60;100)", NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA), PROPORTION_RANGE = c(NA, NA, "(48;52)", 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, "(48;52)", 
        NA, NA, NA, NA, NA, NA, NA), REPEATED_MEASURES_VARS = c(NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_
        ), REPEATED_MEASURES_GOLDSTANDARD = c(NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_, NA_character_, 
        NA_character_, NA_character_, NA_character_), CO_VARS = c(NA, 
        NA, NA, NA, "AGE_0 | SEX_0", "AGE_0 | SEX_0", NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA), 
        END_DIGIT_CHECK = c(FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 
        FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 
        TRUE, TRUE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 
        FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 
        FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 
        FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, 
        FALSE, FALSE), VALUE_LABEL_TABLE = c("LABS_Berlin Hamburg...::97896", 
        NA, "LABS_females males::8a1bf", NA, NA, NA, NA, "LABS_no yes::cde3d", 
        "LABS_excellen good...::dfd3a", NA, "LABS_(-Inf,20 (20,30]...::8d957", 
        "LABS_USR_101 USR_103...::f7ae9", "LABS_USR_121 USR_123...::c993d", 
        NA, NA, NA, NA, NA, "LABS_pre-prim primary...::66ffb", "LABS_single married...::5eb6b", 
        "LABS_no yes::cde3d", NA, "LABS_none vegetari...::ba106", 
        "LABS_never 1-2d a w...::9f0f5", "LABS_no yes::cde3d", "LABS_never 1-2d a w...::9f0f5", 
        NA, NA, "LABS_below 10 [10-30k)...::7c9d1", "LABS_no yes::cde3d", 
        "LABS_no yes::cde3d", NA, "LABS_SR_120 USR_125...::be858", 
        NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, NA, "LABS_(-Inf,20 (20,30]...::8d957", 
        "LABS_females males::8a1bf", NA, "LABS_pre-prim primary...::66ffb", 
        "LABS_no yes::cde3d", "LABS_no yes::cde3d", "LABS_no yes::cde3d", 
        "LABS_no yes::cde3d", "LABS_no yes::cde3d"), MISSING_LIST = c("|", 
        "|", "|", "|", "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "|", "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "|", "|", "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "|", "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "|", "|", "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "|", "|", "99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing", 
        "|", "|", "|", "|", "|"), JUMP_LIST = c("|", "|", "|", "|", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "|", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "|", "|", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "|", "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "88880 = JUMP 88880 | 88890 = JUMP 88890", "88880 = JUMP 88880 | 88890 = JUMP 88890", 
        "|", "|", "88880 = JUMP 88880 | 88890 = JUMP 88890", "|", 
        "|", "88880 = JUMP 88880 | 88890 = JUMP 88890", "|", "|", 
        "|", "|", "|")), version = 2, normalized = TRUE, row.names = c("v00000", 
    "v00001", "v00002", "v00003", "v00004", "v00005", "v00006", "v00007", 
    "v00008", "v00009", "v00010", "v00011", "v00012", "v00013", "v00014", 
    "v00015", "v00016", "v00017", "v00018", "v00019", "v00020", "v00021", 
    "v00022", "v00023", "v00024", "v00025", "v00026", "v00027", "v00028", 
    "v00029", "v00030", "v00031", "v00032", "v00033", "v00034", "v00035", 
    "v00036", "v00037", "v00038", "v00039", "v00040", "v00041", "v00042", 
    "v00103", "v00109", "v01002", "v01003", "v01018", "v10000", "v20000", 
    "v30000", "v40000", "v50000"), class = "data.frame"), ml_error = c("v00004", 
    "v00005", "v00006", "v00007", "v00008", "v00009", "v00010", "v00011", 
    "v00012", "v00014", "v00015", "v00018", "v00019", "v00020", "v00021", 
    "v00022", "v00023", "v00024", "v00025", "v00026", "v00027", "v00028", 
    "v00029", "v00030", "v00031", "v00032", "v00034", "v00035", "v00036", 
    "v00037", "v00038", "v00039", "v00040", "v00041", "v00109", "v01018"
    ), sl_error = c("v00008", "v00010", "v00018", "v00021", "v00023", 
    "v00025", "v00026", "v00027", "v00028", "v00031", "v00034", "v00035", 
    "v00036", "v00037", "v00038", "v00039", "v00040", "v00041", "v00103", 
    "v00109", "v01018"), dt_error = c("v00004", "v00005", "v00009", 
    "v00015"))

---

    c("In the item-level 'meta_data' \"v00004\", \"v00005\", \"v00009\", \"v00015\" may have assigned the wrong 'DATA_TYPE' given 'study_data'", 
    "In the item-level 'meta_data' \"v00008\", \"v00010\", \"v00018\", \"v00021\", \"v00023\", \"v00025\", \"v00026\", \"v00027\", \"v00028\", \"v00031\", \"v00034\", \"v00035\", \"v00036\", \"v00037\", \"v00038\", \"v00039\", \"v00040\", \"v00041\", \"v00103\", \"v00109\", \"v01018\" may have assigned the wrong 'SCALE_LEVEL' given 'study_data'", 
    "In the item-level 'meta_data' \"v00004\", \"v00005\", \"v00006\", \"v00007\", \"v00008\", \"v00009\", \"v00010\", \"v00011\", \"v00012\", \"v00014\", \"v00015\", \"v00018\", \"v00019\", \"v00020\", \"v00021\", \"v00022\", \"v00023\", \"v00024\", \"v00025\", \"v00026\", \"v00027\", \"v00028\", \"v00029\", \"v00030\", \"v00031\", \"v00032\", \"v00034\", \"v00035\", \"v00036\", \"v00037\", \"v00038\", \"v00039\", \"v00040\", \"v00041\", \"v00109\", \"v01018\" may have assigned the wrong 'MISSING_LIST' given 'study_data'"
    )

