# complex_data_types

    Code
      attr(r, "meta_data")[attr(r, "meta_data")[[LABEL]] == "N_CHILD_0", , FALSE]
    Output
         VAR_NAMES     LABEL DATA_TYPE SCALE_LEVEL STANDARDIZED_VOCABULARY_TABLE
      30    v00021 N_CHILD_0   integer       ratio                          <NA>
         MISSING_LIST_TABLE HARD_LIMITS DETECTION_LIMITS SOFT_LIMITS DISTRIBUTION
      30      missing_table    [0; Inf)             <NA>       [0;5]         <NA>
         DECIMALS GROUP_VAR_OBSERVER GROUP_VAR_DEVICE TIME_VAR STUDY_SEGMENT PART_VAR
      30     <NA>       USR_SOCDEM_0             <NA>     <NA>     INTERVIEW   v40000
         VARIABLE_ROLE VARIABLE_ORDER         LONG_LABEL ELEMENT_HOMOGENITY_CHECKTYPE
      30       primary             30 Number of children                         <NA>
         UNIVARIATE_OUTLIER_CHECKTYPE N_RULES LOCATION_METRIC LOCATION_RANGE
      30                         <NA>       4            <NA>           <NA>
         PROPORTION_RANGE REPEATED_MEASURES_VARS REPEATED_MEASURES_GOLDSTANDARD
      30             <NA>                   <NA>                           <NA>
         CO_VARS END_DIGIT_CHECK EXTENDED_DATA_TYPE VALUE_LABEL_TABLE
      30    <NA>           FALSE              count              <NA>
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            MISSING_LIST
      30 99980 = Missing - other reason | 99981 = Missing - exclusion criteria | 99982 = Missing - refusal | 99983 = Missing - not assessable | 99984 = Missing - technical problem | 99985 = Missing - not available (material) | 99986 = Missing - not usable (material) | 99987 = Missing - reason unknown | 99988 = Missing - optional value | 99989 = Deleted - other reason | 99990 = Deleted - contradiction | 99991 = Deleted - value outside limits | 99992 = Inaccurate - other reason | 99993 = Inaccurate - value above detection limit | 99994 = Inaccurate - value below detection limit | 99995 = Data management ongoing
                                       JUMP_LIST
      30 88880 = JUMP 88880 | 88890 = JUMP 88890

---

    Code
      repsum

