# multiplication works

    Code
      md0
    Output
        VAR_NAMES LABEL DATA_TYPE SCALE_LEVEL UNIT MISSING_LIST_TABLE HARD_LIMITS
      1     speed speed   integer     ordinal <NA>               <NA>        <NA>
      2      dist  dist   integer       ratio <NA>               <NA>        <NA>
        CO_VARS GROUP_VAR_OBSERVER GROUP_VAR_DEVICE TIME_VAR PART_VAR STUDY_SEGMENT
      1    <NA>               <NA>             <NA>     <NA>     <NA>          <NA>
      2    <NA>               <NA>             <NA>     <NA>     <NA>          <NA>
        LONG_LABEL LOCATION_RANGE LOCATION_METRIC PROPORTION_RANGE
      1      speed           <NA>            <NA>             <NA>
      2       dist           <NA>            <NA>             <NA>
        UNIVARIATE_OUTLIER_CHECKTYPE N_RULES END_DIGIT_CHECK VALUE_LABEL_TABLE
      1                         <NA>    <NA>           FALSE              <NA>
      2                         <NA>    <NA>           FALSE              <NA>

