% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/000_options.R
\docType{data}
\name{dataquieR.guess_character}
\alias{dataquieR.guess_character}
\title{For metadata guessing, try to guess \link{DATA_TYPE} from the data values}
\description{
By default, the \link{DATA_TYPE} is derived from the R data type of the study
data. However, when data are imported from plain text files, it can be more
appropriate to examine the actual values and infer the data type based on
their content. This option enables that behavior: set
\code{dataquieR.guess_character} to \code{TRUE} to infer data types from the observed
values rather than relying solely on the column’s \code{class} in the data frame.
}
\seealso{
Other options:
\code{\link{dataquieR}},
\code{\link{dataquieR.CONDITIONS_LEVEL_TRHESHOLD}},
\code{\link{dataquieR.CONDITIONS_WITH_STACKTRACE}},
\code{\link{dataquieR.ELEMENT_MISSMATCH_CHECKTYPE}},
\code{\link{dataquieR.ERRORS_WITH_CALLER}},
\code{\link{dataquieR.GAM_for_LOESS}},
\code{\link{dataquieR.MAHALANOBIS_THRESHOLD}},
\code{\link{dataquieR.MAX_LABEL_LEN}},
\code{\link{dataquieR.MAX_LONG_LABEL_LEN}},
\code{\link{dataquieR.MAX_VALUE_LABEL_LEN}},
\code{\link{dataquieR.MESSAGES_WITH_CALLER}},
\code{\link{dataquieR.MULTIVARIATE_OUTLIER_CHECK}},
\code{\link{dataquieR.VALUE_LABELS_htmlescaped}},
\code{\link{dataquieR.WARNINGS_WITH_CALLER}},
\code{\link{dataquieR.acc_loess.exclude_constant_subgroups}},
\code{\link{dataquieR.acc_loess.mark_time_points}},
\code{\link{dataquieR.acc_loess.min_bw}},
\code{\link{dataquieR.acc_loess.min_obs_in_subgroup}},
\code{\link{dataquieR.acc_loess.min_proportion}},
\code{\link{dataquieR.acc_loess.plot_format}},
\code{\link{dataquieR.acc_loess.plot_observations}},
\code{\link{dataquieR.acc_margins_num}},
\code{\link{dataquieR.acc_margins_sort}},
\code{\link{dataquieR.acc_multivariate_outlier.scale}},
\code{\link{dataquieR.acc_shape_or_scale_ci}},
\code{\link{dataquieR.col_con_con_empirical}},
\code{\link{dataquieR.col_con_con_logical}},
\code{\link{dataquieR.convert_to_list_for_lapply}},
\code{\link{dataquieR.debug}},
\code{\link{dataquieR.des_summary_hard_lim_remove}},
\code{\link{dataquieR.dontwrapresults}},
\code{\link{dataquieR.droplevels_ReportSummaryTable}},
\code{\link{dataquieR.dt_adjust}},
\code{\link{dataquieR.fix_column_type_on_read}},
\code{\link{dataquieR.flip_mode}},
\code{\link{dataquieR.force_item_specific_missing_codes}},
\code{\link{dataquieR.force_label_col}},
\code{\link{dataquieR.grading_formats}},
\code{\link{dataquieR.grading_rulesets}},
\code{\link{dataquieR.guess_missing_codes}},
\code{\link{dataquieR.ignore_empty_vars}},
\code{\link{dataquieR.lang}},
\code{\link{dataquieR.lazy_plots}},
\code{\link{dataquieR.lazy_plots_cache}},
\code{\link{dataquieR.lazy_plots_gg_compatibility}},
\code{\link{dataquieR.locale}},
\code{\link{dataquieR.max_cat_resp_var_levels_in_plot}},
\code{\link{dataquieR.max_group_var_levels_in_plot}},
\code{\link{dataquieR.max_group_var_levels_with_violins}},
\code{\link{dataquieR.min_obs_per_group_var_in_plot}},
\code{\link{dataquieR.min_time_points_for_cat_resp_var}},
\code{\link{dataquieR.no_geom_count_in_bin}},
\code{\link{dataquieR.no_overall_in_bin}},
\code{\link{dataquieR.non_disclosure}},
\code{\link{dataquieR.old_factor_handling}},
\code{\link{dataquieR.old_type_adjust}},
\code{\link{dataquieR.precomputeStudyData}},
\code{\link{dataquieR.print_block_load_factor}},
\code{\link{dataquieR.progress_fkt_default}},
\code{\link{dataquieR.progress_msg_fkt_default}},
\code{\link{dataquieR.resume_checkpoint}},
\code{\link{dataquieR.resume_print}},
\code{\link{dataquieR.scale_level_heuristics_control_binaryrecodelimit}},
\code{\link{dataquieR.scale_level_heuristics_control_metriclevels}},
\code{\link{dataquieR.study_data_cache_max}},
\code{\link{dataquieR.study_data_cache_metrics}},
\code{\link{dataquieR.study_data_cache_metrics_env}},
\code{\link{dataquieR.study_data_cache_quick_fill}},
\code{\link{dataquieR.study_data_colnames_case_sensitive}},
\code{\link{dataquieR.testdebug}},
\code{\link{dataquieR.traceback}},
\code{\link{dataquieR.type_adjust_parallel}},
\code{\link{progress_init_fkt}}
}
\concept{options}
