% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.ReportSummaryTable.R
\name{print.ReportSummaryTable}
\alias{print.ReportSummaryTable}
\title{print implementation for the class \code{ReportSummaryTable}}
\usage{
\method{print}{ReportSummaryTable}(
  x,
  relative = lifecycle::deprecated(),
  dt = FALSE,
  fillContainer = FALSE,
  displayValues = FALSE,
  view = TRUE,
  drop = getOption("dataquieR.droplevels_ReportSummaryTable",
    dataquieR.droplevels_ReportSummaryTable_default),
  ...,
  flip_mode = "auto"
)
}
\arguments{
\item{x}{\code{ReportSummaryTable} objects to print}

\item{relative}{deprecated}

\item{dt}{\link{logical} use \code{DT::datatables}, if installed}

\item{fillContainer}{\link{logical} if \code{dt} is \code{TRUE}, control table size,
see \code{DT::datatables}.}

\item{displayValues}{\link{logical} if \code{dt} is \code{TRUE}, also display the actual
values}

\item{view}{\link{logical} if \code{view} is \code{FALSE}, do not print but return the
output, only}

\item{drop}{\link{logical} if \code{drop} is \code{FALSE}, keep unused levels,
see \link{dataquieR.droplevels_ReportSummaryTable}}

\item{...}{not used, yet}

\item{flip_mode}{\link{enum} default | flip | noflip | auto. Should the plot be
in default orientation, flipped, not flipped or
auto-flipped. Not all options are always supported.
In general, this con be controlled by
setting the \code{roptions(dataquieR.flip_mode = ...)}. If
called from \code{dq_report}, you can also pass
\code{flip_mode} to all function calls or set them
specifically using \code{specific_args}.}
}
\value{
the printed object
}
\description{
Use this function to print results objects of the class
\code{ReportSummaryTable}.
}
\seealso{
base::print
}
