% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.dataquieR_summary.R
\name{print.dataquieR_summary}
\alias{print.dataquieR_summary}
\title{Print a \code{dataquieR} summary}
\usage{
\method{print}{dataquieR_summary}(
  x,
  ...,
  grouped_by = c("call_names", "indicator_metric"),
  dont_print = FALSE,
  folder_of_report = NULL,
  vars_to_include = c("study")
)
}
\arguments{
\item{x}{the \code{dataquieR} summary, see \code{\link[=summary]{summary()}} and \code{\link[=dq_report2]{dq_report2()}}}

\item{...}{not yet used}

\item{grouped_by}{define the columns of the resulting matrix. It can be either
"call_names", one column per function,  or "indicator_metric", one column per indicator
or both c("call_names", "indicator_metric"). The last combination is the default}

\item{dont_print}{suppress the actual printing, just return a printable
object derived from \code{x}}

\item{folder_of_report}{a named vector with the location of variable and
\code{call_names}}

\item{vars_to_include}{\code{"study"}, \code{"ssi"} or \code{c("study", "ssi")}. variables
to include}
}
\value{
invisible html object
}
\description{
Print a \code{dataquieR} summary
}
