% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataquieR-package.R
\docType{package}
\name{dataquieR}
\alias{dataquieR}
\alias{dataquieR-package}
\alias{dataquieR-options}
\title{The \code{dataquieR} package about Data Quality in
Epidemiological Research}
\description{
For a quick start please read \link{dq_report2} and maybe the vignettes
or the package's \href{https://dataquality.qihs.uni-greifswald.de/}{website}.
}
\section{Options}{
This package features the following \code{options()}:
\itemize{
\item \link{dataquieR.CONDITIONS_LEVEL_TRHESHOLD}
\item \link{dataquieR.CONDITIONS_WITH_STACKTRACE}
\item \link{dataquieR.ELEMENT_MISSMATCH_CHECKTYPE}
\item \link{dataquieR.ERRORS_WITH_CALLER}
\item \link{dataquieR.GAM_for_LOESS}
\item \link{dataquieR.MAHALANOBIS_THRESHOLD}
\item \link{dataquieR.MAX_LABEL_LEN}
\item \link{dataquieR.MAX_LONG_LABEL_LEN}
\item \link{dataquieR.MAX_VALUE_LABEL_LEN}
\item \link{dataquieR.MESSAGES_WITH_CALLER}
\item \link{dataquieR.MULTIVARIATE_OUTLIER_CHECK}
\item \link{dataquieR.VALUE_LABELS_htmlescaped}
\item \link{dataquieR.WARNINGS_WITH_CALLER}
\item \link{dataquieR.acc_loess.exclude_constant_subgroups}
\item \link{dataquieR.acc_loess.mark_time_points}
\item \link{dataquieR.acc_loess.min_bw}
\item \link{dataquieR.acc_loess.min_obs_in_subgroup}
\item \link{dataquieR.acc_loess.min_proportion}
\item \link{dataquieR.acc_loess.plot_format}
\item \link{dataquieR.acc_loess.plot_observations}
\item \link{dataquieR.acc_margins_num}
\item \link{dataquieR.acc_margins_sort}
\item \link{dataquieR.acc_multivariate_outlier.scale}
\item \link{dataquieR.acc_shape_or_scale_ci}
\item \link{dataquieR.col_con_con_empirical}
\item \link{dataquieR.col_con_con_logical}
\item \link{dataquieR.convert_to_list_for_lapply}
\item \link{dataquieR.debug}
\item \link{dataquieR.des_summary_hard_lim_remove}
\item \link{dataquieR.dontwrapresults}
\item \link{dataquieR.droplevels_ReportSummaryTable}
\item \link{dataquieR.dt_adjust}
\item \link{dataquieR.fix_column_type_on_read}
\item \link{dataquieR.flip_mode}
\item \link{dataquieR.force_item_specific_missing_codes}
\item \link{dataquieR.force_label_col}
\item \link{dataquieR.grading_formats}
\item \link{dataquieR.grading_rulesets}
\item \link{dataquieR.guess_character}
\item \link{dataquieR.guess_missing_codes}
\item \link{dataquieR.ignore_empty_vars}
\item \link{dataquieR.lang}
\item \link{dataquieR.lazy_plots}
\item \link{dataquieR.lazy_plots_cache}
\item \link{dataquieR.lazy_plots_gg_compatibility}
\item \link{dataquieR.locale}
\item \link{dataquieR.max_cat_resp_var_levels_in_plot}
\item \link{dataquieR.max_group_var_levels_in_plot}
\item \link{dataquieR.max_group_var_levels_with_violins}
\item \link{dataquieR.min_obs_per_group_var_in_plot}
\item \link{dataquieR.min_time_points_for_cat_resp_var}
\item \link{dataquieR.no_geom_count_in_bin}
\item \link{dataquieR.no_overall_in_bin}
\item \link{dataquieR.non_disclosure}
\item \link{dataquieR.old_factor_handling}
\item \link{dataquieR.old_type_adjust}
\item \link{dataquieR.precomputeStudyData}
\item \link{dataquieR.print_block_load_factor}
\item \link{dataquieR.resume_checkpoint}
\item \link{dataquieR.resume_print}
\item \link{dataquieR.scale_level_heuristics_control_binaryrecodelimit}
\item \link{dataquieR.scale_level_heuristics_control_metriclevels}
\item \link{dataquieR.study_data_cache_max}
\item \link{dataquieR.study_data_cache_metrics}
\item \link{dataquieR.study_data_cache_metrics_env}
\item \link{dataquieR.study_data_cache_quick_fill}
\item \link{dataquieR.study_data_colnames_case_sensitive}
\item \link{dataquieR.testdebug}
\item \link{dataquieR.traceback}
\item \link{dataquieR.type_adjust_parallel}
}
}

\references{
\itemize{
\item \href{https://joss.theoj.org/papers/10.21105/joss.06581}{JOSS 2024}
\item \href{https://joss.theoj.org/papers/10.21105/joss.03093}{JOSS 2021}
\item \href{https://www.researchsquare.com/article/rs-119457/v1}{DQ_OBS data quality framework}
}
}
\seealso{
Useful links:
\itemize{
  \item \url{https://dataquality.qihs.uni-greifswald.de/}
  \item Report bugs at \url{https://gitlab.com/libreumg/dataquier/-/work_items/issues}
}


Other options:
\code{\link{dataquieR.CONDITIONS_LEVEL_TRHESHOLD}},
\code{\link{dataquieR.CONDITIONS_WITH_STACKTRACE}},
\code{\link{dataquieR.ELEMENT_MISSMATCH_CHECKTYPE}},
\code{\link{dataquieR.ERRORS_WITH_CALLER}},
\code{\link{dataquieR.GAM_for_LOESS}},
\code{\link{dataquieR.MAHALANOBIS_THRESHOLD}},
\code{\link{dataquieR.MAX_LABEL_LEN}},
\code{\link{dataquieR.MAX_LONG_LABEL_LEN}},
\code{\link{dataquieR.MAX_VALUE_LABEL_LEN}},
\code{\link{dataquieR.MESSAGES_WITH_CALLER}},
\code{\link{dataquieR.MULTIVARIATE_OUTLIER_CHECK}},
\code{\link{dataquieR.VALUE_LABELS_htmlescaped}},
\code{\link{dataquieR.WARNINGS_WITH_CALLER}},
\code{\link{dataquieR.acc_loess.exclude_constant_subgroups}},
\code{\link{dataquieR.acc_loess.mark_time_points}},
\code{\link{dataquieR.acc_loess.min_bw}},
\code{\link{dataquieR.acc_loess.min_obs_in_subgroup}},
\code{\link{dataquieR.acc_loess.min_proportion}},
\code{\link{dataquieR.acc_loess.plot_format}},
\code{\link{dataquieR.acc_loess.plot_observations}},
\code{\link{dataquieR.acc_margins_num}},
\code{\link{dataquieR.acc_margins_sort}},
\code{\link{dataquieR.acc_multivariate_outlier.scale}},
\code{\link{dataquieR.acc_shape_or_scale_ci}},
\code{\link{dataquieR.col_con_con_empirical}},
\code{\link{dataquieR.col_con_con_logical}},
\code{\link{dataquieR.convert_to_list_for_lapply}},
\code{\link{dataquieR.debug}},
\code{\link{dataquieR.des_summary_hard_lim_remove}},
\code{\link{dataquieR.dontwrapresults}},
\code{\link{dataquieR.droplevels_ReportSummaryTable}},
\code{\link{dataquieR.dt_adjust}},
\code{\link{dataquieR.fix_column_type_on_read}},
\code{\link{dataquieR.flip_mode}},
\code{\link{dataquieR.force_item_specific_missing_codes}},
\code{\link{dataquieR.force_label_col}},
\code{\link{dataquieR.grading_formats}},
\code{\link{dataquieR.grading_rulesets}},
\code{\link{dataquieR.guess_character}},
\code{\link{dataquieR.guess_missing_codes}},
\code{\link{dataquieR.ignore_empty_vars}},
\code{\link{dataquieR.lang}},
\code{\link{dataquieR.lazy_plots}},
\code{\link{dataquieR.lazy_plots_cache}},
\code{\link{dataquieR.lazy_plots_gg_compatibility}},
\code{\link{dataquieR.locale}},
\code{\link{dataquieR.max_cat_resp_var_levels_in_plot}},
\code{\link{dataquieR.max_group_var_levels_in_plot}},
\code{\link{dataquieR.max_group_var_levels_with_violins}},
\code{\link{dataquieR.min_obs_per_group_var_in_plot}},
\code{\link{dataquieR.min_time_points_for_cat_resp_var}},
\code{\link{dataquieR.no_geom_count_in_bin}},
\code{\link{dataquieR.no_overall_in_bin}},
\code{\link{dataquieR.non_disclosure}},
\code{\link{dataquieR.old_factor_handling}},
\code{\link{dataquieR.old_type_adjust}},
\code{\link{dataquieR.precomputeStudyData}},
\code{\link{dataquieR.print_block_load_factor}},
\code{\link{dataquieR.progress_fkt_default}},
\code{\link{dataquieR.progress_msg_fkt_default}},
\code{\link{dataquieR.resume_checkpoint}},
\code{\link{dataquieR.resume_print}},
\code{\link{dataquieR.scale_level_heuristics_control_binaryrecodelimit}},
\code{\link{dataquieR.scale_level_heuristics_control_metriclevels}},
\code{\link{dataquieR.study_data_cache_max}},
\code{\link{dataquieR.study_data_cache_metrics}},
\code{\link{dataquieR.study_data_cache_metrics_env}},
\code{\link{dataquieR.study_data_cache_quick_fill}},
\code{\link{dataquieR.study_data_colnames_case_sensitive}},
\code{\link{dataquieR.testdebug}},
\code{\link{dataquieR.traceback}},
\code{\link{dataquieR.type_adjust_parallel}},
\code{\link{progress_init_fkt}}
}
\author{
\strong{Maintainer}: Stephan Struckmann \email{stephan.struckmann@uni-greifswald.de} (\href{https://orcid.org/0000-0002-8565-7962}{ORCID})

Authors:
\itemize{
  \item Stephan Struckmann \email{stephan.struckmann@uni-greifswald.de} (\href{https://orcid.org/0000-0002-8565-7962}{ORCID})
  \item Elisa Kasbohm \email{elisa.kasbohm@uni-greifswald.de} (\href{https://orcid.org/0000-0001-5261-538X}{ORCID})
  \item Elena Salogni \email{elena.salogni@uni-greifswald.de} (\href{https://orcid.org/0009-0007-3767-7145}{ORCID})
  \item Joany Marino \email{joany.marino@uni-greifswald.de} (\href{https://orcid.org/0000-0002-4657-3758}{ORCID})
  \item Adrian Richter \email{richtera@uni-greifswald.de} (\href{https://orcid.org/0000-0002-3372-2021}{ORCID})
  \item Carsten Oliver Schmidt \email{carsten.schmidt@uni-greifswald.de} (\href{https://orcid.org/0000-0001-5266-9396}{ORCID})
}

Other contributors:
\itemize{
  \item University Medicine Greifswald [copyright holder]
  \item German Research Foundation (DFG SCHM 2744/3-1, SCHM 2744/9-1, SCHM 2744/3-4) [funder]
  \item National Research Data Infrastructure for Personal Health Data: (NFDI 13/1) [funder]
  \item European Union’s Horizon 2020 programme (euCanSHare, grant agreement No. 825903) [funder]
}

}
\concept{options}
\keyword{internal}
