% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/000_globs.R
\name{WELL_KNOWN_META_VARIABLE_NAMES}
\alias{WELL_KNOWN_META_VARIABLE_NAMES}
\alias{VAR_NAMES}
\alias{LABEL}
\alias{DATA_TYPE}
\alias{SCALE_LEVEL}
\alias{UNIT}
\alias{VALUE_LABELS}
\alias{VALUE_LABEL_TABLE}
\alias{MISSING_LIST}
\alias{JUMP_LIST}
\alias{MISSING_LIST_TABLE}
\alias{HARD_LIMITS}
\alias{DETECTION_LIMITS}
\alias{SOFT_LIMITS}
\alias{CONTRADICTIONS}
\alias{DISTRIBUTION}
\alias{DECIMALS}
\alias{DATA_ENTRY_TYPE}
\alias{END_DIGIT_CHECK}
\alias{CO_VARS}
\alias{GROUP_VAR_OBSERVER}
\alias{GROUP_VAR_DEVICE}
\alias{KEY_OBSERVER}
\alias{KEY_DEVICE}
\alias{TIME_VAR}
\alias{TIME_VAR_END}
\alias{KEY_DATETIME}
\alias{PART_VAR}
\alias{STUDY_SEGMENT}
\alias{KEY_STUDY_SEGMENT}
\alias{VARIABLE_ROLE}
\alias{VARIABLE_ORDER}
\alias{LONG_LABEL}
\alias{SOFT_LIMIT_LOW}
\alias{SOFT_LIMIT_UP}
\alias{HARD_LIMIT_LOW}
\alias{HARD_LIMIT_UP}
\alias{DETECTION_LIMIT_LOW}
\alias{DETECTION_LIMIT_UP}
\alias{INCL_SOFT_LIMIT_LOW}
\alias{INCL_SOFT_LIMIT_UP}
\alias{INCL_HARD_LIMIT_LOW}
\alias{INCL_HARD_LIMIT_UP}
\alias{LOCATION_RANGE}
\alias{LOCATION_METRIC}
\alias{PROPORTION_RANGE}
\alias{LOCATION_LIMIT_LOW}
\alias{LOCATION_LIMIT_UP}
\alias{INCL_LOCATION_LIMIT_LOW}
\alias{INCL_LOCATION_LIMIT_UP}
\alias{PROPORTION_LIMIT_LOW}
\alias{PROPORTION_LIMIT_UP}
\alias{INCL_PROPORTION_LIMIT_LOW}
\alias{INCL_PROPORTION_LIMIT_UP}
\alias{RECODE_CASES}
\alias{RECODE_CONTROL}
\alias{GRADING_RULESET}
\alias{STANDARDIZED_VOCABULARY_TABLE}
\alias{DATAFRAMES}
\alias{ENCODING}
\alias{UNIVARIATE_OUTLIER_CHECKTYPE}
\alias{N_RULES}
\alias{EXTENDED_DATA_TYPE}
\alias{COMPUTED_VARIABLE_ROLE}
\title{Well-known metadata column names, names of metadata columns}
\usage{
WELL_KNOWN_META_VARIABLE_NAMES
}
\description{
names of the variable attributes in the metadata frame holding
the names of the respective observers, devices, lower limits for plausible
values, upper limits for plausible values, lower limits for allowed values,
upper limits for allowed values, the variable name (column name, e.g.
v0020349) used in the study data,  the variable name used for processing
(readable name, e.g. RR_DIAST_1) and in parameters of the QA-Functions, the
variable label, variable long label, variable short label, variable data
type (see also \link{DATA_TYPES}), re-code for definition of lists of event
categories, missing lists and jump lists as CSV strings. For valid units
see \link{UNITS}.
}
\details{
all entries of this list will be mapped to the package's exported NAMESPACE
environment directly, i.e. they are available directly by their names too:
\itemize{
\item \link{VAR_NAMES}
\item \link{LABEL}
\item \link{DATA_TYPE}
\item \link{SCALE_LEVEL}
\item \link{UNIT}
\item \link{VALUE_LABELS}
\item \link{VALUE_LABEL_TABLE}
\item \link{MISSING_LIST}
\item \link{JUMP_LIST}
\item \link{MISSING_LIST_TABLE}
\item \link{HARD_LIMITS}
\item \link{DETECTION_LIMITS}
\item \link{SOFT_LIMITS}
\item \link{CONTRADICTIONS}
\item \link{DISTRIBUTION}
\item \link{DECIMALS}
\item \link{DATA_ENTRY_TYPE}
\item \link{END_DIGIT_CHECK}
\item \link{CO_VARS}
\item \link{GROUP_VAR_OBSERVER}
\item \link{GROUP_VAR_DEVICE}
\item \link{KEY_OBSERVER}
\item \link{KEY_DEVICE}
\item \link{TIME_VAR}
\item \link{TIME_VAR_END}
\item \link{KEY_DATETIME}
\item \link{PART_VAR}
\item \link{STUDY_SEGMENT}
\item \link{KEY_STUDY_SEGMENT}
\item \link{VARIABLE_ROLE}
\item \link{VARIABLE_ORDER}
\item \link{LONG_LABEL}
\item \link{SOFT_LIMIT_LOW}
\item \link{SOFT_LIMIT_UP}
\item \link{HARD_LIMIT_LOW}
\item \link{HARD_LIMIT_UP}
\item \link{DETECTION_LIMIT_LOW}
\item \link{DETECTION_LIMIT_UP}
\item \link{INCL_SOFT_LIMIT_LOW}
\item \link{INCL_SOFT_LIMIT_UP}
\item \link{INCL_HARD_LIMIT_LOW}
\item \link{INCL_HARD_LIMIT_UP}
\item \link{LOCATION_RANGE}
\item \link{LOCATION_METRIC}
\item \link{PROPORTION_RANGE}
\item \link{LOCATION_LIMIT_LOW}
\item \link{LOCATION_LIMIT_UP}
\item \link{INCL_LOCATION_LIMIT_LOW}
\item \link{INCL_LOCATION_LIMIT_UP}
\item \link{PROPORTION_LIMIT_LOW}
\item \link{PROPORTION_LIMIT_UP}
\item \link{INCL_PROPORTION_LIMIT_LOW}
\item \link{INCL_PROPORTION_LIMIT_UP}
\item \link{RECODE_CASES}
\item \link{RECODE_CONTROL}
\item \link{GRADING_RULESET}
\item \link{STANDARDIZED_VOCABULARY_TABLE}
\item \link{DATAFRAMES}
\item \link{ENCODING}
\item \link{UNIVARIATE_OUTLIER_CHECKTYPE}
\item \link{N_RULES}
\item \link{EXTENDED_DATA_TYPE}
\item \link{COMPUTED_VARIABLE_ROLE}
}
}
\examples{
print(WELL_KNOWN_META_VARIABLE_NAMES$VAR_NAMES)
# print(VAR_NAMES) # should usually also work
}
\seealso{
\link{meta_data_segment} for \code{STUDY_SEGMENT}

\href{https://dataquality.qihs.uni-greifswald.de/VIN_Item_Level_Metadata.html}{online}

Other UNITS:
\code{\link{UNITS}},
\code{\link{UNIT_IS_COUNT}},
\code{\link{UNIT_PREFIXES}},
\code{\link{UNIT_PREFIX_FACTORS}},
\code{\link{UNIT_SOURCES}}
}
\concept{UNITS}
\alias{variable attribute}
