% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.dataquieR_result.R
\name{print.DataSlot}
\alias{print.DataSlot}
\title{Print a \code{DataSlot} object}
\usage{
\method{print}{DataSlot}(x, ...)
}
\arguments{
\item{x}{the object}

\item{...}{not used}
}
\value{
see print
}
\description{
Print a \code{DataSlot} object
}
