% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/000_globs.R
\name{CODE_CLASSES}
\alias{CODE_CLASSES}
\title{types of value codes}
\usage{
CODE_CLASSES
}
\description{
types of value codes
}
