% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dq_report.R
\name{dq_report}
\alias{dq_report}
\title{Generate a full DQ report}
\usage{
dq_report(...)
}
\arguments{
\item{...}{Deprecated}
}
\value{
Deprecated
}
\description{
Deprecated
}
