% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prep_meta_data_v1_to_item_level_meta_data.R
\name{prep_meta_data_v1_to_item_level_meta_data}
\alias{prep_meta_data_v1_to_item_level_meta_data}
\title{Convert item-level metadata from v1.0 to v2.0}
\usage{
prep_meta_data_v1_to_item_level_meta_data(
  item_level = "item_level",
  verbose = TRUE,
  label_col = LABEL,
  cause_label_df,
  meta_data = item_level
)
}
\arguments{
\item{item_level}{\link{data.frame} the old item-level-metadata}

\item{verbose}{\link{logical} display all estimated decisions, defaults to \code{TRUE},
except if called in a \link{dq_report2} pipeline.}

\item{label_col}{\link{variable attribute} the name of the column in the metadata
with labels of variables}

\item{cause_label_df}{\link{data.frame} missing code table, see \link{cause_label_df}.
Optional. If this argument is given,
you can add missing code tables.}

\item{meta_data}{\link{data.frame} old name for \code{item_level}}
}
\value{
\link{data.frame} the updated metadata
}
\description{
This function is idempotent..
}
\details{
The \code{options("dataquieR.force_item_specific_missing_codes")} (default
\code{FALSE}) tells the system, to always fill in \code{res_vars} columns to the
\code{MISSING_LIST_TABLE}, even, if the column already exists, but is empty.
}
