% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta_data_env.R
\name{meta_data_env}
\alias{meta_data_env}
\alias{.meta_data_env}
\title{\code{.meta_data_env} -- an environment for easy metadata access}
\usage{
.meta_data_env
}
\description{
used by the dq_report2-pipeline
}
\seealso{
\code{meta_data_env_id_vars()} \code{meta_data_env_co_vars()}
\code{meta_data_env_time_vars()} \code{meta_data_env_group_vars()}
}
\keyword{internal}
